/*
 * EdmondsReloaded.java
 *
 * Created on May 17, 2004, 9:11 PM
 */

import java.util.Vector ;
/**
 * This extends the Edmonds class to allow for changes in 
 * edge capacity
 */
public class EdmondsReloaded extends Edmonds {
    private Vector paths ;
    
    
    
    /** Creates a new instance of EdmondsReloaded */
    public EdmondsReloaded(Graph g) {
        super(g) ;
        paths = new Vector() ;
    }
    
    
    /** Runs the algorithm on the graph. The algorithm can either be run step wise
     * , or until it completes. In this implemation, one step is the addition of one augmenting
     * path.
     * @param cont if true, the algorithm runs to completion. if false, it will run step -wise.
     * @return a result object
     * @throws Exception if an error occurs within the algorithm
     */    
    public Result run(boolean cont) throws Exception {
                
	BFS bfs = new BFS(g) ;
        Path p ;
                
	// START MAIN LOOP
	do {
            // do BFS
            bfs.search(g.getSource()) ;
	
            // check for an augmenting path
            p = PathReloaded.fromGraph(g) ;

            if (p == null ) {
		// no remaining augmenting paths to the sink
		// time to finish
                done = true ;
                break ;
            }
            
            // store this path for later
            paths.add(p) ;
            
            // loop thru all edges in path
            for (int i =0 ; i < p.length() ; i++ ) {
                Edge e = p.getEdge(i) ;

		// set new flow
		e.setFlow( e.getFlow() + p.getBottleneck() ) ;

		// set flow for the oppositte edge
		Edge e2 = e.getComplement() ;
		e2.setFlow(e2.getFlow() - p.getBottleneck() ) ;
            }

	} while (cont) ;
                              
        Result r = new Result(calcFlow(), p == null ) ;
        r.setPath(p) ;
        return r ;	
    }
    
    
    /**
     * Notify the algorithm that the capacity of the edge has changed.
     * @param e the edge that has changed
     */
    void change(Edge e) {
        int overflow = e.getFlow() - e.getCap() ;
        
        // until we still have extra flow in the edge
        for(int i= 0; overflow >0; i++) {
            PathReloaded p = (PathReloaded) paths.get(i) ;
			// if p contains e
            if(p.contains(e)) {
				// reduce flow thru the path
                int red = Math.min(overflow, p.getBottleneck() ) ;
                p.reduceFlowByAmount(red) ;
                overflow = e.getFlow() - e.getCap() ;
                
            }
        }
        
    }
   
} // end class
