/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


import javax.swing.JFileChooser ;
import java.io.File ;
import java.util.Vector ;
import java.util.Hashtable ;
import java.util.Enumeration ; 
import javax.swing.JOptionPane ;

public class Main extends javax.swing.JFrame {
    
    /** Creates new form  */
    public Main() {
        initComponents();
        vd = new VertexDialog(this, true ) ;
        vd.setLocationRelativeTo(this); 
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        jFileChooser1 = new javax.swing.JFileChooser();
        jPanel1 = new javax.swing.JPanel();
        statusLabel = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jToolBar1 = new javax.swing.JToolBar();
        resetButton = new javax.swing.JButton();
        stepButton = new javax.swing.JButton();
        solveButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        algorCombo = new javax.swing.JComboBox();
        jPanel3 = new javax.swing.JPanel();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        jMenuItem2 = new javax.swing.JMenuItem();

        setTitle("Simulator");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        statusLabel.setFont(new java.awt.Font("Dialog", 0, 10));
        statusLabel.setText("Status Bar...");
        jPanel1.add(statusLabel);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        jPanel2.setLayout(new java.awt.BorderLayout());

        jPanel2.setMinimumSize(new java.awt.Dimension(600, 550));
        jPanel2.setPreferredSize(new java.awt.Dimension(600, 550));

        resetButton.setText("Reset");
        resetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetButtonActionPerformed(evt);
            }
        });

        jToolBar1.add(resetButton);

        stepButton.setText("Step");
        stepButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stepButtonActionPerformed(evt);
            }
        });

        jToolBar1.add(stepButton);

        solveButton.setText("Solve");
        solveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                solveButtonActionPerformed(evt);
            }
        });

        jToolBar1.add(solveButton);

        jLabel1.setForeground(new java.awt.Color(0, 0, 0));
        jLabel1.setText("Algorithm:");
        jLabel1.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(1, 10, 1, 10)));
        jToolBar1.add(jLabel1);

        algorCombo.setBackground(new java.awt.Color(255, 255, 255));
        algorCombo.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Edmonds-Karp", "Relabel-to-Front" }));
        algorCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                algorComboActionPerformed(evt);
            }
        });

        jToolBar1.add(algorCombo);

        jPanel2.add(jToolBar1, java.awt.BorderLayout.NORTH);

        jPanel3.setLayout(new AbsoluteLayout());

        jPanel3.setBackground(new java.awt.Color(255, 255, 255));
        jPanel2.add(jPanel3, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);

        jMenu1.setText("File");
        jMenuItem1.setText("Open...");
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem1ActionPerformed(evt);
            }
        });

        jMenu1.add(jMenuItem1);
        jMenu1.add(jSeparator1);
        jMenuItem2.setText("Exit");
        jMenuItem2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem2ActionPerformed(evt);
            }
        });

        jMenu1.add(jMenuItem2);
        jMenuBar1.add(jMenu1);
        setJMenuBar(jMenuBar1);

        pack();
    }

    /**
     * called when the selected item in the combo box chanages
     */
    private void algorComboActionPerformed(java.awt.event.ActionEvent evt) {
        // work out what was selected 
        if(algorCombo.getSelectedIndex() == 0 ) {
            algorithm = EDMONDS ;
            if( g != null) {
                mfa = new EdmondsReloaded(g) ;
            }
        } else {
            algorithm = RELABEL ;
            if( g != null) {
                mfa = new Relabel(g) ;
            }
        }
               
        // notify gVertices of the algorithm change
        if ( vertexHash != null) {
           for(int i =0 ; i < g.getNVertices() ; i++) {
                gVertex gV = (gVertex) vertexHash.get(g.getVertex(i) ) ;
                gV.setAlgor(algorithm); 
           }
        }
               
        // reset view
        reset() ;
    }

    /***
     * called when the user clicks solve
     */
    private void solveButtonActionPerformed(java.awt.event.ActionEvent evt) {
        Result r = null ;
        try {
             r = mfa.run( true) ;   
             done(r) ;
        } catch ( Exception e) {
               System.out.println("BAD BAD BAD") ;
        }
        
        for(int i =0 ; i < g.getNVertices() ; i++) {
           (  (gVertex) vertexHash.get(g.getVertex(i) )  ).updateProp(); 
        }
        jPanel3.repaint() ;
    }

    /**
     * displays a mesg about the max flow, and disabled buttons
     */
    private void done(Result r) {
          stepButton.setEnabled(false) ;
          solveButton.setEnabled(false) ;
          
          if( algorithm == EDMONDS)
            JOptionPane.showMessageDialog(this, "No more paths found, max flow is " + r.getFlow() ); 
          else 
            JOptionPane.showMessageDialog(this, "No more vertices to discharge, max flow is " + r.getFlow() );        
    }
    
    /**
     * called when the user clicks on step
     */
    private void stepButtonActionPerformed(java.awt.event.ActionEvent evt) {
 
        boolean done = false ;
        Result r ; 
        
        try {
            r = mfa.run( false) ;      
        } catch ( Exception e) {
                System.out.println(e.toString() )  ;
                e.printStackTrace() ;
                
               System.out.println("BAD BAD BAD") ;
               return  ; 
        }
        
        // reset edge colors
        if(edgeHash != null) {
             for (Enumeration e = edgeHash.elements() ; e.hasMoreElements() ;) {
                     ((gEdge) e.nextElement()).setAugPath(false)   ;
             }
        }
        
        // check if a path was found
        if(r.isDone()) {
            done(r) ;
          
        } else if( algorithm == EDMONDS) {
            // path exists, highlight 
            // for each edge in path, look up gEdge object an mark as being in the Augmenting path
            Path p = r.getPath() ;
            
            for(int i=0; i < p.length() ; i++ ) {
                ( (gEdge) edgeHash.get(p.getEdge(i)) ).setAugPath(true) ;
            }  
        } else {
            // used relabel to front
            // remove focus from old vertex
            if( old != null) {
                old.setFocus(false) ;
            }
            
            // give focus to new vertex, and store in old
            if( r.getVertex() != null) {
                gVertex newV = (gVertex) vertexHash.get( r.getVertex() )  ;
                newV.setFocus(true) ;
                old = newV ;
            }
        }
        
        // update properties for all vertices
        for(int i =0 ; i < g.getNVertices() ; i++) {
           (  (gVertex) vertexHash.get(g.getVertex(i) )  ).updateProp(); 
        }
        
        // repaint the panel
        jPanel3.repaint() ;
    }

    /**
     *  called when the reset button is clicked
     */
    private void resetButtonActionPerformed(java.awt.event.ActionEvent evt) {
        reset() ;
    }

    /**
     * resets everything
     * called either when the reset button is pressed, of if the algorithm changes
     */
    private void reset() {
        // reset the algorithm
        if (mfa != null) mfa.reset() ;
        
        // remove focus 
        if(old != null) {
            old.setFocus(false) ;
        }
        
        // repaint and enable buttons again
        jPanel3.repaint() ;
        stepButton.setEnabled(true) ;
        solveButton.setEnabled(true) ;  
    }
    
    /**
     * called when the open option is selected from the menu
     */
    private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {
        int returnVal = jFileChooser1.showOpenDialog(this) ;
        
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = jFileChooser1.getSelectedFile();
          
            System.out.println(file.getAbsolutePath() ) ;
            
            loadGraphFromFile(file) ;
        } else {
            // was cancelled do nothing
        }
        
    }

    /**
     * called when exit is seleted from the menu
     */
    private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {
        System.exit(0) ;
    }
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {
        System.exit(0);
    }
    
    
    /**
     * Loads a graph from a file
     *
     */
    private void loadGraphFromFile(File file) {
       
        // remove the old vertices 
        if(vertexHash != null) {
             for (Enumeration e = vertexHash.elements() ; e.hasMoreElements() ;) {
                     jPanel3.remove((gVertex) e.nextElement())  ;
             }
        }
        
        // remove old edges
        if(edgeHash != null) {
             for (Enumeration e = edgeHash.elements() ; e.hasMoreElements() ;) {
                     jPanel3.remove((gEdge) e.nextElement())  ;
             }
        }
              
        // attempt to parse the file
        try { 
            g = Parser.parse(file.getAbsolutePath() ) ;
            g.normalise() ;
        } catch (Exception e) {
            // should do dialog here
            JOptionPane.showMessageDialog(this, "Could not parse input file:" + e.toString() , "Error",  JOptionPane.ERROR_MESSAGE  );
            return ;
        }
        
        // enable buttons
        stepButton.setEnabled(true) ;
        solveButton.setEnabled(true) ;
        
        // give the gVertex class a reference to the graph
        gVertex.setGraph(g) ;
        
        // make new hashtables, these map low level objects (vertex, edge) to 
        // their graphical counterparts (gVertex, gEdge) so we can looks them up in 
        // constant time
        vertexHash =  new Hashtable() ;
        edgeHash = new Hashtable() ;
        
        int angle = 360 / g.getNVertices() ;
        
        // now make a gVertex for each vertex 
        for(int i =0 ; i < g.getNVertices() ; i++) {
            gVertex gV = new gVertex(g.getVertex(i), this ) ;
            // add to hash
            vertexHash.put(g.getVertex(i) , gV) ;
            // add to panel
            
            int x = 250 +  (int) (200 * Math.cos(Math.toRadians(angle * i))) ;
            int y = 250 +  (int) (200 * Math.sin(Math.toRadians(angle * i))) ;
            jPanel3.add(gV, new AbsoluteConstraints(x , y , -1, -1) ) ;
        }
        
        // repaint the panel now, we need to do this so the gVertices are in their correct 
        // positions before we attempt to draw the edges
        pack() ;
        jPanel3.repaint() ;
        
        // now we make all the gEdge objects 
        // foreach vertex
        for (int i =0 ; i < g.getNVertices() ; i++ ) {
            gVertex  gV = (gVertex) vertexHash.get(g.getVertex(i) ) ;
            AdjacencyList al = g.getAdjList(i) ;
            gV.setAlgor(algorithm) ;
            
            // foreach edge leading away from this vertex
            for(int j=0 ; j<al.length() ; j++) {
                // get the target vertex
                gVertex target =  (gVertex) vertexHash.get( al.getVertex(j) ) ;
                // make  a new gEdge object
                gEdge e = new gEdge( gV , target , al.getEdge(j) ) ;
                
                // add to edges hash
                edgeHash.put(al.getEdge(j), e) ;
                // add to panel
                jPanel3.add(e, new AbsoluteConstraints(e.getX(), e.getY(), -1, -1) ) ;
            }
        }
     
        // now repaint the panel to do the edges as well
        pack() ;
        jPanel3.repaint() ;
        
        // make a new solver object
        if(algorithm == EDMONDS ) {
            mfa = new EdmondsReloaded(g) ; ;
        } else {
            mfa = new Relabel(g) ;
        }

    }
    
    public void vertexClicked(Vertex v) {
		if(algorithm != EDMONDS) 
			return ;

		AdjacencyList al = g.getAdjList(v) ;
		// popup window, sent al
		//
		Edge e = vd.show(al, v) ;

		// if a source, we may need to update the size of the link to the super source
	  	if( e != null && v.isSource() ) {
			Edge src = null ;
			int maxOut = 0 ;
			for(int i =0; i<al.length() ; i++ ) {
				if(al.getEdge(i).getCap() > 0) {
					maxOut += al.getEdge(i).getCap() ;
				}
				if(al.getVertex(i) == g.getSource()) {
					src = al.getEdge(i).getComplement() ;
				}
			}
			src.setCap(maxOut) ;
			  
		}

		// return edge and new cap
		if( e != null ) {
                // is the user changed it
                EdmondsReloaded er = (EdmondsReloaded) mfa ;
				er.change(e) ;
				for(int i =0 ; i < g.getNVertices() ; i++) {
           			(  (gVertex) vertexHash.get(g.getVertex(i) )  ).updateProp(); 
        		}

		        jPanel3.repaint() ;        		
        }
		stepButton.setEnabled(true) ;
        solveButton.setEnabled(true) ;  

	}
	
   
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new Main().show();
    }
    
    private gEdge e ;
    private Graph g ;
    private MaxFlowAlgorithm mfa ;
    private Hashtable vertexHash ;
    private Hashtable edgeHash ;
    private gVertex old ;
    private VertexDialog vd ;
    
    public static final int EDMONDS   = 0;
    public static final int RELABEL   = 1;
    private int algorithm = EDMONDS ;   // the algorithm in use
 
    
    // Variables declaration
    private javax.swing.JLabel jLabel1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JButton stepButton;
    private javax.swing.JLabel statusLabel;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JButton solveButton;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JFileChooser jFileChooser1;
    private javax.swing.JComboBox algorCombo;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JButton resetButton;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenuBar jMenuBar1;
    // End of variables declaration
    
}
