import java.io.*;
import java.util.StringTokenizer;

/** This class parses a file into a graph object
 */
public class Parser {
    
    /** Parse the contents of the file, and return a graph object
     * @param file the file to parse
     * @return a graph object from the contents of the file
     * @throws Exception if the parsing fails
     */
    public static Graph parse(String file) throws Exception {
        
        try {
            FileReader fr = new FileReader(file) ;
            BufferedReader br = new BufferedReader(fr) ;
            
            // skip over comments
            String s = br.readLine() ;
            while( s.startsWith("//") ) {
                s = br.readLine() ;
            } ;
            
            // get numer of vertices
            int size = Integer.parseInt(s.trim() ) ;
            int node = 0 ;
            Vertex[] v = new Vertex[size]  ;
            AdjacencyList[] adjList = new AdjacencyList[size] ;
            int[] links = new int[size] ;
            
            // make new vertices
            for(int i = 0 ; i < size ; i++) {
                v[i] = new Vertex(String.valueOf(i) ) ;
            }
            
            while( (s = br.readLine()) != null  && node < size) {
                // check for // here and skip
                if (s.startsWith("//") )
                    continue ;
                
                boolean isSink = true ;
                
                adjList[node] =	new AdjacencyList() ;
                
                StringTokenizer st = new StringTokenizer(s) ;
                
                int n = 0 ;
                while(st.hasMoreTokens()) {
                    String token = st.nextToken() ;
                    int cap = Integer.parseInt(token) ;
                    
                    if(cap != 0) {
                        adjList[node].addAdjacentVertex(v[n], new Edge(cap)  ) ;
                        isSink = false ;
                        links[n] = 1 ;
                    }
                    n++ ;
                }
                
                if (isSink)
                    v[node].setSink() ;
                
                node++ ;
            }
            
            Graph g =  new Graph(v, adjList) ;
            
            // work out which vertices are sources
            for ( int i = 0; i < size ; i++ ) {
                if( links[i] == 0 ) {
                    v[i].setSource() ;
                }
            }
            
            return g ;
            
        } catch (Exception e) {
            System.out.println(e) ;
            e.printStackTrace() ;
            throw new Exception("parsing Graph failed") ;
        }
    }
    
} // end class
