/*
 * PathReloaded.java
 *
 * Created on May 18, 2004, 4:57 PM
 */

/***
 * This extends to path class to allow for seaching and reducing 
 * all edges in the path
 */

public class PathReloaded extends Path {
    
    /** Creates a new instance of PathReloaded */
    public PathReloaded(Graph g) {
        super(g) ;
    }
    
	/***
	 * Returns true if the path contains the edge, 
	 * false otherwise
	 * @param edge the edge to check for
	 */

    boolean contains(Edge e) {
        for( int i = 0; i < edges.size()  ; i++) {
            if (e == (Edge) edges.get(i) ) 
                return true ;
		}
		return false; 
    }
    
	/***
	 * Reduces all edges in this path, by the amount flow
	 * @param flow, the amount to reduce the paths by
	 */
    void reduceFlowByAmount(int flow) {
        for( int i = 0; i < edges.size()  ; i++) {
            Edge e = (Edge) edges.get(i)  ;
            e.setFlow( e.getFlow() - flow ) ;
        }
    }
    
    /** Takes a Graph that has had a BFS performed on it,
     * and returns the path from the source to the sink, or null if no path exists
     * @param g the graph
     * @return A Path object, or null if no path exists
     */      
    public static Path fromGraph(Graph g) {
        if (g.getSink().getParent() == null ) {
            return null ;
        }
	
        // ok now we know a path exists
        // follow it back to the source
        Vertex tempV = g.getSink() ;
        Path p = new PathReloaded(g) ;
		
        //  goto sink node
        //  step back, adding edges to paths
        //  find bottleneck

        while (tempV != g.getSource()) {
            Edge e = g.getAdjList(tempV.getParent()).getEdgeToVertex(tempV) ;
            p.addEdge(e) ;
            if( p.bottleneck == -1 || ((p.bottleneck > e.getRes() ) && (e.getRes() != -1) ) ) 
                p.bottleneck = e.getRes() ;
            tempV = tempV.getParent() ;
        }
        return p ;
    }
    
}
