
/** this class repsents the a result returned from a MaxFlowAlgorithm
 */
public class Result {
    Path p ;
    Vertex v ;
    
    int flow ;
    boolean done = false ;
    
    /** Creates a new instance of Result
     * @param flow the current flow thru the graph
     * @param done if the algorithm has terminated
     */
    public Result(int  flow, boolean done ) {
        this.done = done ;
        this.flow = flow ;
    }
    
    /** set the augmenting path object, used for the Edmonds-Karp algorithm
     * @param p the path to set
     */    
    public void setPath(Path p ) {
        this.p =p  ;
    }   
    
    /** returns the augmenting path, or null if none was found or the relabel algorithm was used
     * @return the augmenting path
     */    
    public Path getPath() {
        return p ;
    }
    
    /** sets the last vertex to be discharged (used in the relabel to front algorithm)
     * @param v the vertex to set
     */    
    public void setVertex(Vertex v ) {
        this.v =v  ;
    }   
    
    /** returns the vertex that was last discharged (used in the relabel to front algorithm)
     * @return the last vertex to be discharged
     */    
    public Vertex getVertex() {
        return v ;
    }
    
    /** returns the flow thru the network, this is the maximium flow
     * if the algorithm has terminated
     * @return the flow
     */    
    public int getFlow() {
        return flow ;
    }
    
    /** checks if the algorithm has terminated
     * @return true is the algorithm has terminated, false otherwise
     */    
    public boolean isDone() {
        return done ;
    }
}
