

/** This class represents a vertex in a graph
 */
public class Vertex {
    
    private String label;
    private boolean visited;	// = false if not visited, true otherwise
    private Vertex parent;		// parent vertex of the current vertex
    private int depth = -1;		// for BFS
    private char color = 'W' ;      // for BFS
    private boolean source = false ;
    private boolean sink = false ;
    private int height ;            // for relabel to front
    private int excess ;            // for relabel to front


    /** Create a new vertex with the specified label
     * @param label the label for this vertex
     */
    public Vertex(String label) {
        this.label = label;
    }


    /** Returns the label of this vertex
     * @return the label for this vertex
     */
    public String getLabel() {
        return label;
    }
    

    /** Returns the color of this vertex (use for BFS)
     * Either 'W' , 'G' or 'B'
     * @return a character represeting the color of this vertex
     */
    public char getColor() {
        return color ;
    }
    
    /** Sets the color for this vertex, should be either 'W' , 'G', or 'B'
     * @param color a character representing the color
     */
    public void setColor(char color) {
        this.color = color ;
    }
       
    
    /** set the height of this vertex (used for relabel-to-front)
     * @param h the new height of this vertex
     */
    public void setHeight(int h) {
        height = h ;
    }
    
    /** returns the height of this vertes
     * @return the height of the vertex
     */
    public int getHeight() {
        return height ;
    }
    
    /** sets the excess in this vertex (used for relabel-to-front)
     * @param e the excess
     */
    public void setExcess(int e) {
        excess = e ;
    }
    
    /** returns the excess in this vertex (used for relabel-to-front)
     * @return the excess in the vertex
     */
    public int getExcess() {
        return excess ;
    }
    
    /** set the parent of this vertex (used for BFS)
     * @param v the parent of this vertex
     */
    public void setParent(Vertex v) {
        parent = v;
    }
    
    /** Returns the parent of this vertex
     * @return A Vertex representing the parent of the vertex
     */
    public Vertex getParent() {
        return parent;
    }
    
    
    // we might not need depth anymore
    /** sets the depth of this vertex (used in BFS)
     * @param depth the depth of this vertex
     */
    public void setDepth(int depth) {
        this.depth = depth;
    }
    
    /** returns the depth of this vertex (used for BFS)
     * @return the depth of this vertex
     */
    public int getDepth() {
        return depth;
    }
    
    /** Checks if this vertex is an ancestor of the specified one
     * @param v the Vertex to check
     * @return true if this is an ancestor, false otherwise
     */
    public boolean isParent(Vertex v) {
        if (v.parent == null)
            return false;
        else if (v.parent == this)
            return true;
        else
            return isParent(v.parent);
    }
    
    /** Sets this vertex to a source
     */
    public void setSource() {
        source = true ;
    }
    
    /** checks if this vertex is a source
     * @return true if the vertex is a source, false otherwise
     */
    public boolean isSource() {
        return source ;
    }
    
    /** sets this vertex to a sink
     */
    public void setSink() {
        sink = true ;
    }
    
    /** checks if this vertex is a sink
     * @return true if the vertex is a sink, false otherwise
     */
    public boolean isSink() {
        return sink ;
    }
    
    /** to String method :)
     * @return a string representing this Vertex
     */
    public String toString() {
        return label ;
    }
}

