

import java.awt.event.* ;
import java.util.Vector ; 

/** This is a graphical representation of a vertex. It extends a javax.swing.jLabel
 */
public class gVertex extends javax.swing.JLabel implements MouseMotionListener, MouseListener {
    
    private static Graph g ;
    
    private boolean dragging = false ;
    private int x ;
    private int y ;
    private Vertex v ;
    private Vector edges ;
    private int algorithm ; 
	private Main m ;
    
    private static final int INSET = 8 ;
    

    /** creates a new gVertex
     * @param v the Vertex object that this represents
     */    
    public gVertex(Vertex v, Main m) {
        super() ;
        this.v = v;
		this.m = m ;
        addMouseMotionListener(this);
        addMouseListener(this) ;
        setText(v.getLabel() ) ;
        
        edges = new Vector() ; 
        if(v.isSource() ) {
            setBorder(new javax.swing.border.CompoundBorder(new javax.swing.border.EtchedBorder(new java.awt.Color(102, 153, 255), new java.awt.Color(0, 0, 204)), new javax.swing.border.EmptyBorder(new java.awt.Insets(INSET, INSET, INSET, INSET))) ) ;
        } else if (v.isSink() ) {
            setBorder(new javax.swing.border.CompoundBorder(new javax.swing.border.EtchedBorder(new java.awt.Color(255, 153, 102), new java.awt.Color(255, 0, 0)), new javax.swing.border.EmptyBorder(new java.awt.Insets(INSET, INSET, INSET, INSET))) ) ;           
        } else {
            setBorder(new javax.swing.border.CompoundBorder(new javax.swing.border.EtchedBorder(), new javax.swing.border.EmptyBorder(new java.awt.Insets(INSET, INSET, INSET, INSET))) ) ;
        }
        this.setOpaque(true) ;
        updateProp() ;
    }
    
    /** add an edge to the list of edges that get notified when this object moves
     * @param gE the edge to add
     */
    public void addEdge(gEdge gE) {
        edges.add(gE) ;
    }
    
    /** updates the tooltip for this vertex
     */    
    public void updateProp() {
        AdjacencyList al = g.getAdjList( g.getVertex(v) ) ;
        int sum =0;
        // sum up the flow comming into this vertex
        for(int j=0 ; j<al.length() ; j++ ) {
            int flow = al.getEdge(j).getFlow() ;
           
            // if flow going out
            if(flow > 0 )
                sum += al.getEdge(j).getFlow() ;
            // else , for the sink we have to count up incomming flow
            else if (flow < 0 && g.getSink() == v)
                sum += -al.getEdge(j).getFlow() ;
        }  
        
       if(algorithm == Main.EDMONDS) {
        setToolTipText("Flow: " + sum);
       } else {
        setToolTipText("Height: "  + v.getHeight() + ", Excess: " + v.getExcess() + ", Flow: " + sum ); 
       }
    }
    
    /** marks this vertex as the last one discharged
     */    
    public void setFocus(boolean focus) {
        if(focus) {
             setBorder(new javax.swing.border.CompoundBorder(new javax.swing.border.EtchedBorder(new java.awt.Color(51, 255, 51), new java.awt.Color(0, 204, 0)), new javax.swing.border.EmptyBorder(new java.awt.Insets(INSET, INSET, INSET, INSET))) ) ;
        } else {
            if(v.isSource() ) {
                setBorder(new javax.swing.border.CompoundBorder(new javax.swing.border.EtchedBorder(new java.awt.Color(102, 153, 255), new java.awt.Color(0, 0, 204)), new javax.swing.border.EmptyBorder(new java.awt.Insets(INSET, INSET, INSET, INSET))) ) ;
            } else if (v.isSink() ) {
                setBorder(new javax.swing.border.CompoundBorder(new javax.swing.border.EtchedBorder(new java.awt.Color(255, 153, 102), new java.awt.Color(255, 0, 0)), new javax.swing.border.EmptyBorder(new java.awt.Insets(INSET, INSET, INSET, INSET))) ) ;           
            } else {
                setBorder(new javax.swing.border.CompoundBorder(new javax.swing.border.EtchedBorder(), new javax.swing.border.EmptyBorder(new java.awt.Insets(INSET, INSET, INSET, INSET))) ) ;
            }
         }
    }
    
    /** sets the algorithm in use
     */    
    public void setAlgor(int a) {
        algorithm = a ;
        updateProp(); 
    }
    
    
    /** mouse event handlers ..
     * @param mouseEvent the mouse event
     */    
    public void mouseDragged(java.awt.event.MouseEvent mouseEvent) {
      if(! dragging) {
          dragging = true ;
          x = mouseEvent.getX() ;
          y = mouseEvent.getY() ;
      }
    
      this.setLocation( this.getX() + mouseEvent.getX() - x , this.getY() + mouseEvent.getY() - y) ;
      repaint() ;
      
      for(int i=0; i< edges.size() ; i++) {
          ((gEdge) edges.get(i)).update() ;
      }
        
    }

    /** mouse event handlers ..
     * @param mouseEvent the mouse event
     */  
    public void mouseMoved(java.awt.event.MouseEvent mouseEvent) {
    }
    
    
    /** mouse event handlers ..
     * @param mouseEvent the mouse event
     */
    public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {   
		m.vertexClicked(v) ;
    }
   
    /** mouse event handlers ..
     * @param mouseEvent the mouse event
     */  
    public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
    }
    
    /** mouse event handlers ..
     * @param mouseEvent the mouse event
     */  
    public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
    }
    
    /** mouse event handlers ..
     * @param mouseEvent the mouse event
     */  
    public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
    }
    
    
    /** mouse event handlers ..
     * @param mouseEvent the mouse event
     */  
    public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
        dragging = false ;
        
    }
    
    
    public static void setGraph(Graph graph) {
        g = graph ;
    }
    

    
}
